@@ -47,19 +47,30 @@ default_pathologies = [  'Atelectasis',
                 ]
 
 thispath = os.path.dirname(os.path.realpath(__file__))
+datapath = os.path.join(thispath,"data")
 
 # this is for caching small things for speed
 _cache_dict = {}
 
-def normalize(sample, maxval):
+def normalize(img, maxval, reshape=False):
     """Scales images to be roughly [-1024 1024]."""
     
-    if sample.max() > maxval:
-        raise Exception("max image value ({}) higher than expected bound ({}).".format(sample.max(), maxval))
+    if img.max() > maxval:
+        raise Exception("max image value ({}) higher than expected bound ({}).".format(img.max(), maxval))
     
-    sample = (2 * (sample.astype(np.float32) / maxval) - 1.) * 1024
-    #sample = sample / np.std(sample)
-    return sample
+    img = (2 * (img.astype(np.float32) / maxval) - 1.) * 1024
+
+    if reshape:
+        # Check that images are 2D arrays
+        if len(img.shape) > 2:
+            img = img[:, :, 0]
+        if len(img.shape) < 2:
+            print("error, dimension lower than 2 for image")
+
+        # add color channel
+        img = img[None, :, :] 
+    
+    return img
 
 def relabel_dataset(pathologies, dataset, silent=False):
     """
@@ -105,7 +116,7 @@ class Dataset():
             raise Exception("csvpath must be a file")
         
     
-class Merge_Dataset(Dataset):
+class MergeDataset(Dataset):
     def __init__(self, datasets, seed=0, label_concat=False):
         super(Merge_Dataset, self).__init__()
         np.random.seed(seed)  # Reset the seed so all runs are the same.
@@ -142,7 +153,7 @@ class Merge_Dataset(Dataset):
         except:
             print("Could not merge dataframes (.csv not available):", sys.exc_info()[0])
         
-        self.csv = self.csv.reset_index()
+        self.csv = self.csv.reset_index(drop=True)
 
     def string(self):
         s = self.__class__.__name__ + " num_samples={}\n".format(len(self))
@@ -162,6 +173,11 @@ class Merge_Dataset(Dataset):
         item["source"] = self.which_dataset[idx]
         return item
         
+
+# alias so it is backwards compatable
+Merge_Dataset = MergeDataset
+
+
 class FilterDataset(Dataset):
     def __init__(self, dataset, labels=None):
         super(FilterDataset, self).__init__()
@@ -232,16 +248,14 @@ class NIH_Dataset(Dataset):
     https://academictorrents.com/details/e615d3aebce373f1dc8bd9d11064da55bdadede0
     """
     def __init__(self, imgpath, 
-                 csvpath=os.path.join(thispath, "Data_Entry_2017_v2020.csv.gz"),
-                 bbox_list_path=os.path.join(thispath, "BBox_List_2017.csv.gz"),
+                 csvpath=os.path.join(datapath, "Data_Entry_2017_v2020.csv.gz"),
+                 bbox_list_path=os.path.join(datapath, "BBox_List_2017.csv.gz"),
                  views=["PA"],
                  transform=None, 
                  data_aug=None, 
                  nrows=None, 
                  seed=0,
-                 pure_labels=False, 
                  unique_patients=True,
-                 normalize=True,
                  pathology_masks=False):
         
         super(NIH_Dataset, self).__init__()
@@ -260,11 +274,9 @@ class NIH_Dataset(Dataset):
         
         self.pathologies = sorted(self.pathologies)
         
-        self.normalize = normalize
         # Load data
         self.check_paths_exist()
         self.csv = pd.read_csv(self.csvpath, nrows=nrows)
-        self.MAXVAL = 255  # Range [0 255]
         
         if type(views) is not list:
             views = [views]
@@ -273,10 +285,6 @@ class NIH_Dataset(Dataset):
         self.csv["view"] = self.csv['View Position']
         self.csv = self.csv[self.csv["view"].isin(self.views)]
         
-        # Remove multi-finding images.
-        if pure_labels:
-            self.csv = self.csv[~self.csv["Finding Labels"].str.contains("\|")]
-        
         if unique_patients:
             self.csv = self.csv.groupby("Patient ID").first()
         
@@ -289,7 +297,7 @@ class NIH_Dataset(Dataset):
                skiprows=1)
         
         # change label name to match
-        self.pathology_maskscsv["Finding Label"][self.pathology_maskscsv["Finding Label"] == "Infiltrate"] = "Infiltration"
+        self.pathology_maskscsv.loc[self.pathology_maskscsv["Finding Label"] == "Infiltrate", "Finding Label"] = "Infiltration"
         self.csv["has_masks"] = self.csv["Image Index"].isin(self.pathology_maskscsv["Image Index"])
         ####### pathology masks ########    
             
@@ -327,17 +335,8 @@ class NIH_Dataset(Dataset):
         img_path = os.path.join(self.imgpath, imgid)
         #print(img_path)
         img = imread(img_path)
-        if self.normalize:
-            img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
 
         transform_seed = np.random.randint(2147483647)
         
@@ -409,8 +408,8 @@ class RSNA_Pneumonia_Dataset(Dataset):
     """
     def __init__(self, 
                  imgpath, 
-                 csvpath=os.path.join(thispath, "kaggle_stage_2_train_labels.csv.zip"),
-                 dicomcsvpath=os.path.join(thispath, "kaggle_stage_2_train_images_dicom_headers.csv.gz"),
+                 csvpath=os.path.join(datapath, "kaggle_stage_2_train_labels.csv.zip"),
+                 dicomcsvpath=os.path.join(datapath, "kaggle_stage_2_train_images_dicom_headers.csv.gz"),
                  views=["PA"],
                  transform=None, 
                  data_aug=None, 
@@ -418,7 +417,6 @@ class RSNA_Pneumonia_Dataset(Dataset):
                  seed=0,
                  pure_labels=False, 
                  unique_patients=True,
-                 normalize=True,
                  pathology_masks=False,
                  extension=".jpg"):
 
@@ -433,8 +431,6 @@ class RSNA_Pneumonia_Dataset(Dataset):
         
         self.pathologies = sorted(self.pathologies)
         
-        self.normalize=normalize
-        
         self.extension = extension
         self.use_pydicom=( extension == ".dcm" ) 
         
@@ -452,8 +448,6 @@ class RSNA_Pneumonia_Dataset(Dataset):
         
         self.csv = self.csv.join(self.dicomcsv, on="patientId")
         
-        self.MAXVAL = 255  # Range [0 255]
-        
         if type(views) is not list:
             views = [views]
         self.views = views
@@ -501,17 +495,8 @@ class RSNA_Pneumonia_Dataset(Dataset):
             img=pydicom.filereader.dcmread(img_path).pixel_array
         else:
             img = imread(img_path)
-        if self.normalize:
-            img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]                
+            
+        sample["img"] = normalize(img, maxval=255, reshape=True)              
                                
         transform_seed = np.random.randint(2147483647)
         
@@ -580,15 +565,14 @@ class NIH_Google_Dataset(Dataset):
     """
     
     def __init__(self, imgpath, 
-                 csvpath=os.path.join(thispath, "google2019_nih-chest-xray-labels.csv.gz"), 
+                 csvpath=os.path.join(datapath, "google2019_nih-chest-xray-labels.csv.gz"), 
                  views=["PA"],
                  transform=None, 
                  data_aug=None, 
                  nrows=None, 
                  seed=0,
                  pure_labels=False, 
-                 unique_patients=True,
-                 normalize=True):
+                 unique_patients=True):
 
         super(NIH_Google_Dataset, self).__init__()
         np.random.seed(seed)  # Reset the seed so all runs are the same.
@@ -600,13 +584,10 @@ class NIH_Google_Dataset(Dataset):
                             "Nodule or mass"]
         
         self.pathologies = sorted(self.pathologies)
-        
-        self.normalize = normalize
 
         # Load data
         self.csvpath = csvpath
         self.csv = pd.read_csv(self.csvpath, nrows=nrows)
-        self.MAXVAL = 255  # Range [0 255]
         
         if type(views) is not list:
             views = [views]
@@ -632,6 +613,8 @@ class NIH_Google_Dataset(Dataset):
         
         # rename pathologies
         self.pathologies = np.char.replace(self.pathologies, "Airspace opacity", "Lung Opacity")
+        self.pathologies = np.char.replace(self.pathologies, "Nodule or mass", "Nodule/Mass")
+        self.pathologies = list(self.pathologies)
 
     def string(self):
         return self.__class__.__name__ + " num_samples={} views={} data_aug={}".format(len(self), self.views, self.data_aug)
@@ -640,29 +623,25 @@ class NIH_Google_Dataset(Dataset):
         return len(self.labels)
 
     def __getitem__(self, idx):
+        
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         imgid = self.csv['Image Index'].iloc[idx]
         img_path = os.path.join(self.imgpath, imgid)
         #print(img_path)
         img = imread(img_path)
-        if self.normalize:
-            img = normalize(img, self.MAXVAL)  
 
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        img = img[None, :, :]                    
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
 
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
             
-        return {"img":img, "lab":self.labels[idx], "idx":idx}
+        return sample
     
     
 class PC_Dataset(Dataset):
@@ -684,7 +663,7 @@ class PC_Dataset(Dataset):
     https://academictorrents.com/details/96ebb4f92b85929eadfb16761f310a6d04105797
     """
     def __init__(self, imgpath, 
-                 csvpath=os.path.join(thispath, "PADCHEST_chest_x_ray_images_labels_160K_01.02.19.csv.gz"), 
+                 csvpath=os.path.join(datapath, "PADCHEST_chest_x_ray_images_labels_160K_01.02.19.csv.gz"), 
                  views=["PA"],
                  transform=None, 
                  data_aug=None,
@@ -701,9 +680,10 @@ class PC_Dataset(Dataset):
                             "Cardiomegaly", "Nodule", "Mass", "Hernia","Fracture", 
                             "Granuloma", "Flattened Diaphragm", "Bronchiectasis",
                             "Aortic Elongation", "Scoliosis", 
-                            "Hilar Enlargement", "Support Devices" , "Tuberculosis",
+                            "Hilar Enlargement", "Tuberculosis",
                             "Air Trapping", "Costophrenic Angle Blunting", "Aortic Atheromatosis",
-                            "Hemidiaphragm Elevation"]
+                            "Hemidiaphragm Elevation", 
+                            "Support Devices", "Tube'"] # the Tube' is intentional
         
         self.pathologies = sorted(self.pathologies)
         
@@ -723,6 +703,7 @@ class PC_Dataset(Dataset):
                                         "pulmonary artery enlargement"]
         mapping["Support Devices"] = ["device",
                                       "pacemaker"]
+        mapping["Tube'"] = ["stent'"] ## the ' is to select findings which end in that word
         
         self.imgpath = imgpath
         self.transform = transform
@@ -732,7 +713,6 @@ class PC_Dataset(Dataset):
         
         self.check_paths_exist()
         self.csv = pd.read_csv(self.csvpath, low_memory=False)
-        self.MAXVAL = 65535
 
         # standardize view names
         self.csv.loc[self.csv["Projection"].isin(["AP_horizontal"]),"Projection"] = "AP Supine"
@@ -758,11 +738,15 @@ class PC_Dataset(Dataset):
                    "216840111366964012373310883942009117084022290_00-064-025.png",
                    "216840111366964012283393834152009033102258826_00-059-087.png",
                    "216840111366964012373310883942009170084120009_00-097-074.png",
-                   "216840111366964012819207061112010315104455352_04-024-184.png"]
+                   "216840111366964012819207061112010315104455352_04-024-184.png",
+                   "216840111366964012819207061112010306085429121_04-020-102.png"]
         self.csv = self.csv[~self.csv["ImageID"].isin(missing)]
         
         if unique_patients:
             self.csv = self.csv.groupby("PatientID").first().reset_index()
+            
+        # filter out age < 10 (paper published 2019)
+        self.csv = self.csv[(2019-self.csv.PatientBirth > 10)]
         
         # Get our classes.
         self.labels = []
@@ -776,11 +760,13 @@ class PC_Dataset(Dataset):
         self.labels = np.asarray(self.labels).T
         self.labels = self.labels.astype(np.float32)
         
+        self.pathologies[self.pathologies.index("Tube'")] = "Tube"
+        
         ########## add consistent csv values
         
         # offset_day_int
         dt = pd.to_datetime(self.csv["StudyDate_DICOM"], format="%Y%m%d")
-        self.csv["offset_day_int"] = dt.astype(np.int)// 10**9 // 86400
+        self.csv["offset_day_int"] = dt.astype(int)// 10**9 // 86400
         
         # patientid
         self.csv["patientid"] = self.csv["PatientID"].astype(str)
@@ -793,27 +779,23 @@ class PC_Dataset(Dataset):
 
     def __getitem__(self, idx):
 
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         imgid = self.csv['ImageID'].iloc[idx]
         img_path = os.path.join(self.imgpath,imgid)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)   
         
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        img = img[None, :, :]                    
+        sample["img"] = normalize(img, maxval=65535, reshape=True)               
                                
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
             
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
 
-        return {"img":img, "lab":self.labels[idx], "idx":idx}
+        return sample
 
 class CheX_Dataset(Dataset):
     """
@@ -826,12 +808,18 @@ class CheX_Dataset(Dataset):
     Dataset website here:
     https://stanfordmlgroup.github.io/competitions/chexpert/
     """
-    def __init__(self, imgpath, csvpath, views=["PA"], transform=None, data_aug=None,
-                 flat_dir=True, seed=0, unique_patients=True):
+    def __init__(self, 
+                 imgpath, 
+                 csvpath=os.path.join(datapath, "chexpert_train.csv.gz"),
+                 views=["PA"], 
+                 transform=None, 
+                 data_aug=None,
+                 flat_dir=True, 
+                 seed=0, 
+                 unique_patients=True):
 
         super(CheX_Dataset, self).__init__()
         np.random.seed(seed)  # Reset the seed so all runs are the same.
-        self.MAXVAL = 255
         
         self.pathologies = ["Enlarged Cardiomediastinum",
                             "Cardiomegaly",
@@ -867,7 +855,7 @@ class CheX_Dataset(Dataset):
         self.csv = self.csv[self.csv["view"].isin(self.views)] # Select the view 
          
         if unique_patients:
-            self.csv["PatientID"] = self.csv["Path"].str.extract(pat = '(patient\d+)')
+            self.csv["PatientID"] = self.csv["Path"].str.extract(pat = r'(patient\d+)')
             self.csv = self.csv.groupby("PatientID").first().reset_index()
                    
         # Get our classes.
@@ -875,7 +863,8 @@ class CheX_Dataset(Dataset):
         self.labels = []
         for pathology in self.pathologies:
             if pathology in self.csv.columns:
-                self.csv.loc[healthy, pathology] = 0
+                if pathology != "Support Devices":
+                    self.csv.loc[healthy, pathology] = 0
                 mask = self.csv[pathology]
                 
             self.labels.append(mask.values)
@@ -912,28 +901,24 @@ class CheX_Dataset(Dataset):
 
     def __getitem__(self, idx):
         
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         imgid = self.csv['Path'].iloc[idx]
         imgid = imgid.replace("CheXpert-v1.0-small/","")
         img_path = os.path.join(self.imgpath, imgid)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)      
-        
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
 
-        # Add color channel
-        img = img[None, :, :]
+        sample["img"] = normalize(img, maxval=255, reshape=True)
 
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
             
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
 
-        return {"img":img, "lab":self.labels[idx], "idx":idx}
+        return sample
     
 class MIMIC_Dataset(Dataset):
     """
@@ -951,7 +936,6 @@ class MIMIC_Dataset(Dataset):
 
         super(MIMIC_Dataset, self).__init__()
         np.random.seed(seed)  # Reset the seed so all runs are the same.
-        self.MAXVAL = 255
         
         self.pathologies = ["Enlarged Cardiomediastinum",
                             "Cardiomegaly",
@@ -1029,30 +1013,26 @@ class MIMIC_Dataset(Dataset):
 
     def __getitem__(self, idx):
         
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         subjectid = str(self.csv.iloc[idx]["subject_id"])
         studyid = str(self.csv.iloc[idx]["study_id"])
         dicom_id = str(self.csv.iloc[idx]["dicom_id"])
         
         img_path = os.path.join(self.imgpath, "p" + subjectid[:2], "p" + subjectid, "s" + studyid, dicom_id + ".jpg")
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)      
         
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        img = img[None, :, :]
+        sample["img"] = normalize(img, maxval=255, reshape=True)
 
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
             
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
 
-        return {"img":img, "lab":self.labels[idx], "idx":idx}
+        return sample
     
 class Openi_Dataset(Dataset):
     """
@@ -1069,9 +1049,9 @@ class Openi_Dataset(Dataset):
     https://academictorrents.com/details/5a3a439df24931f410fac269b87b050203d9467d
     """
     def __init__(self, imgpath, 
-                 xmlpath=os.path.join(thispath, "NLMCXR_reports.tgz"), 
-                 dicomcsv_path=os.path.join(thispath, "nlmcxr_dicom_metadata.csv.gz"),
-                 tsnepacsv_path=os.path.join(thispath, "nlmcxr_tsne_pa.csv.gz"),
+                 xmlpath=os.path.join(datapath, "NLMCXR_reports.tgz"), 
+                 dicomcsv_path=os.path.join(datapath, "nlmcxr_dicom_metadata.csv.gz"),
+                 tsnepacsv_path=os.path.join(datapath, "nlmcxr_tsne_pa.csv.gz"),
                  filter_pa=True,
                  transform=None, data_aug=None, 
                  nrows=None, seed=0,
@@ -1126,7 +1106,6 @@ class Openi_Dataset(Dataset):
                     samples.append(sample)
        
         self.csv = pd.DataFrame(samples)
-        self.MAXVAL = 255  # Range [0 255]
             
         self.dicom_metadata = pd.read_csv(dicomcsv_path, index_col="imageid", low_memory=False)
 
@@ -1179,28 +1158,25 @@ class Openi_Dataset(Dataset):
         return len(self.labels)
 
     def __getitem__(self, idx):
+        
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         imageid = self.csv.iloc[idx].imageid
         img_path = os.path.join(self.imgpath,imageid + ".png")
         #print(img_path)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        img = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
 
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
             
-        return {"img":img, "lab":self.labels[idx], "idx":idx}
+        return sample
 
 class COVID19_Dataset(Dataset):
     """
@@ -1220,7 +1196,7 @@ class COVID19_Dataset(Dataset):
     def __init__(self, 
                  imgpath=os.path.join(thispath, "covid-chestxray-dataset", "images"), 
                  csvpath=os.path.join(thispath, "covid-chestxray-dataset", "metadata.csv"), 
-                 semantic_masks_v7labs_lungs_path=os.path.join(thispath, "data" , "semantic_masks_v7labs_lungs.zip"),
+                 semantic_masks_v7labs_lungs_path=os.path.join(datapath, "semantic_masks_v7labs_lungs.zip"),
                  views=["PA", "AP"],
                  transform=None, 
                  data_aug=None, 
@@ -1242,7 +1218,6 @@ class COVID19_Dataset(Dataset):
         # Load data
         self.csvpath = csvpath
         self.csv = pd.read_csv(self.csvpath, nrows=nrows)
-        self.MAXVAL = 255  # Range [0 255]
 
         # Keep only the frontal views.
         #idx_pa = self.csv["view"].isin(["PA", "AP", "AP Supine"])
@@ -1292,16 +1267,8 @@ class COVID19_Dataset(Dataset):
         img_path = os.path.join(self.imgpath, imgid)
         #print(img_path)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         transform_seed = np.random.randint(2147483647)
         
@@ -1395,9 +1362,6 @@ class NLMTB_Dataset(Dataset):
         self.labels = self.csv["label"].values.reshape(-1,1)
         self.pathologies = ["Tuberculosis"]
         self.views = views
-        
-        
-        self.MAXVAL = 255
 
     def string(self):
         return self.__class__.__name__ + " num_samples={} views={} data_aug={}".format(len(self), self.views, self.data_aug)
@@ -1406,28 +1370,25 @@ class NLMTB_Dataset(Dataset):
         return len(self.labels)
 
     def __getitem__(self, idx):
+        
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        
         item = self.csv.iloc[idx]
         img_path = os.path.join(self.imgpath, "CXR_png", item["fname"])
         #print(img_path)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        img = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         if self.transform is not None:
-            img = self.transform(img)
+            sample["img"] = self.transform(sample["img"])
 
         if self.data_aug is not None:
-            img = self.data_aug(img)
+            sample["img"] = self.data_aug(sample["img"])
             
-        return {"img":img, "lab":self.labels[idx], "idx":idx}    
+        return sample  
 
 class SIIM_Pneumothorax_Dataset(Dataset):
     """
@@ -1440,24 +1401,23 @@ class SIIM_Pneumothorax_Dataset(Dataset):
     
     def __init__(self, 
                  imgpath, 
-                 csvpath, 
+                 csvpath=os.path.join(datapath, "siim-pneumothorax-train-rle.csv.gz"), 
                  transform=None, 
                  data_aug=None, 
                  seed=0,
                  unique_patients=True,
-                 masks=False):
+                 pathology_masks=False):
 
         super(SIIM_Pneumothorax_Dataset, self).__init__()
         np.random.seed(seed)  # Reset the seed so all runs are the same.
         self.imgpath = imgpath
         self.transform = transform
         self.data_aug = data_aug
-        self.masks = masks
+        self.pathology_masks = pathology_masks
                 
         # Load data
         self.csvpath = csvpath
         self.csv = pd.read_csv(self.csvpath)
-        self.MAXVAL = 255  # Range [0 255]
         
         self.pathologies = ["Pneumothorax"]
         
@@ -1498,26 +1458,18 @@ class SIIM_Pneumothorax_Dataset(Dataset):
         #print(img_path)
         img = pydicom.filereader.dcmread(img_path).pixel_array
         #img = imread(img_path)
-        img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         transform_seed = np.random.randint(2147483647)
         
-        if self.masks:
+        if self.pathology_masks:
             sample["pathology_masks"] = self.get_pathology_mask_dict(imgid, sample["img"].shape[2])
             
         if self.transform is not None:
             random.seed(transform_seed)
             sample["img"] = self.transform(sample["img"])
-            if self.masks:
+            if self.pathology_masks:
                 for i in sample["pathology_masks"].keys():
                     random.seed(transform_seed)
                     sample["pathology_masks"][i] = self.transform(sample["pathology_masks"][i])
@@ -1525,7 +1477,7 @@ class SIIM_Pneumothorax_Dataset(Dataset):
         if self.data_aug is not None:
             random.seed(transform_seed)
             sample["img"] = self.data_aug(sample["img"])
-            if self.masks:
+            if self.pathology_masks:
                 for i in sample["pathology_masks"].keys():
                     random.seed(transform_seed)
                     sample["pathology_masks"][i] = self.data_aug(sample["pathology_masks"][i])
@@ -1584,7 +1536,7 @@ class VinBrain_Dataset(Dataset):
     https://www.kaggle.com/c/vinbigdata-chest-xray-abnormalities-detection
     """
     def __init__(self, imgpath, 
-                 csvpath, 
+                 csvpath=os.path.join(datapath, "vinbigdata-train.csv.gz"), 
                  views=None,
                  transform=None, 
                  data_aug=None,
@@ -1622,12 +1574,13 @@ class VinBrain_Dataset(Dataset):
         self.mapping["Pleural_Thickening"] = ["Pleural thickening"]
         self.mapping["Effusion"] = ["Pleural effusion"]
         
-        self.normalize = normalize
         # Load data
         self.check_paths_exist()
         self.rawcsv = pd.read_csv(self.csvpath)
         self.csv = pd.DataFrame(self.rawcsv.groupby("image_id")["class_name"].apply(lambda x: "|".join(np.unique(x))))
         
+        self.csv["has_masks"] = self.csv.class_name != "No finding"
+        
         self.labels = []
         for pathology in self.pathologies:
             mask = self.csv["class_name"].str.lower().str.contains(pathology.lower())
@@ -1677,17 +1630,8 @@ class VinBrain_Dataset(Dataset):
         else:
             raise Exception("Unknown Photometric Interpretation mode")
             
-        if self.normalize:
-            img = normalize(img, 2**float(bitdepth))
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]
+            
+        sample["img"] = normalize(img, maxval=2**float(bitdepth), reshape=True)
         
         transform_seed = np.random.randint(2147483647)
         
@@ -1779,7 +1723,7 @@ class StonyBrookCOVID_Dataset(Dataset):
         
         date_col = self.csv["Exam_DateTime"].str.split("_",expand=True)[0]
         dt = pd.to_datetime(date_col, format="%Y%m%d")
-        self.csv["offset_day_int"] = dt.astype(np.int)// 10**9 // 86400
+        self.csv["offset_day_int"] = dt.astype(int)// 10**9 // 86400
         
         # patientid
         self.csv["patientid"] = self.csv["Subject_ID"].astype(str)
@@ -1799,16 +1743,8 @@ class StonyBrookCOVID_Dataset(Dataset):
         img_path = os.path.join(self.imgpath, str(idx) + ".jpg")
         #print(img_path)
         img = imread(img_path)
-        img = normalize(img, self.MAXVAL)  
-
-        # Check that images are 2D arrays
-        if len(img.shape) > 2:
-            img = img[:, :, 0]
-        if len(img.shape) < 2:
-            print("error, dimension lower than 2 for image")
-
-        # Add color channel
-        sample["img"] = img[None, :, :]                    
+        
+        sample["img"] = normalize(img, maxval=255, reshape=True)
                                
         transform_seed = np.random.randint(2147483647)
             
@@ -1820,9 +1756,73 @@ class StonyBrookCOVID_Dataset(Dataset):
             random.seed(transform_seed)
             sample["img"] = self.data_aug(sample["img"])
             
-        return sample
-
+        return sample 
     
+class ObjectCXR_Dataset(Dataset):
+    """
+    https://jfhealthcare.github.io/object-CXR/
+
+    https://academictorrents.com/details/fdc91f11d7010f7259a05403fc9d00079a09f5d5
+    """
+    def __init__(self, imgzippath, 
+                 csvpath, 
+                 transform=None, 
+                 data_aug=None,
+                 seed=0):
+
+        super(ObjectCXR_Dataset, self).__init__()
+
+        np.random.seed(seed)  # Reset the seed so all runs are the same.
+        self.imgzippath = imgzippath
+        self.csvpath = csvpath
+        self.transform = transform
+        self.data_aug = data_aug
+        self.views = []
+        self.pathologies = ['Foreign Object']
+
+        # Load data
+        self.csv = pd.read_csv(self.csvpath)
+
+        self.labels = []
+        self.labels.append(~self.csv["annotation"].isnull())
+        self.labels = np.asarray(self.labels).T
+        self.labels = self.labels.astype(np.float32)
+
+        self.csv = self.csv.reset_index()
+
+        self.csv["has_masks"] = ~self.csv["annotation"].isnull()
+
+        self.imgzip = zipfile.ZipFile(self.imgzippath)
+
+    def string(self):
+        return self.__class__.__name__ + " num_samples={} views={} data_aug={}".format(len(self), self.views, self.data_aug)
+
+    def __len__(self):
+        return len(self.labels)
+
+    def __getitem__(self, idx):
+
+        sample = {}
+        sample["idx"] = idx
+        sample["lab"] = self.labels[idx]
+        imgid = self.csv.iloc[idx]["image_name"]
+
+        with zipfile.ZipFile(self.imgzippath).open("train/" + imgid) as file:
+            sample["img"] = imageio.imread(file.read())
+        
+        sample["img"] = normalize(sample["img"], maxval=255, reshape=True)
+
+        transform_seed = np.random.randint(2147483647)
+
+        if self.transform is not None:
+            random.seed(transform_seed)
+            sample["img"] = self.transform(sample["img"])
+
+        if self.data_aug is not None:
+            random.seed(transform_seed)
+            sample["img"] = self.data_aug(sample["img"])
+
+        return sample
     
     
 class ToPILImage(object):
@@ -1939,8 +1939,14 @@ class CovariateDataset(Dataset):
         all_1_pos = np.setdiff1d(all_1_pos, train_1_pos)
 
         if verbose:
-            print("TRAIN: neg={}, pos={}".format(len(train_0_neg)+len(train_1_neg),
-                                                 len(train_0_pos)+len(train_1_pos)))
+            print("TRAIN (ratio={:.2}): neg={}, pos={}, d1_pos/neg={}/{}, d2_pos/neg={}/{}".format(
+                   ratio,
+                   len(train_0_neg)+len(train_1_neg),
+                   len(train_0_pos)+len(train_1_pos),
+                   len(train_0_pos),
+                   len(train_0_neg),
+                   len(train_1_pos),
+                   len(train_1_neg)))
 
         # VALID
         valid_0_neg = np.random.choice(
@@ -1959,8 +1965,14 @@ class CovariateDataset(Dataset):
         all_1_pos = np.setdiff1d(all_1_pos, valid_1_pos)
 
         if verbose:
-            print("VALID: neg={}, pos={}".format(len(valid_0_neg)+len(valid_1_neg),
-                                                 len(valid_0_pos)+len(valid_1_pos)))
+            print("VALID (ratio={:.2}): neg={}, pos={}, d1_pos/neg={}/{}, d2_pos/neg={}/{}".format(
+                   1-ratio,
+                   len(valid_0_neg)+len(valid_1_neg),
+                   len(valid_0_pos)+len(valid_1_pos),
+                   len(valid_0_pos),
+                   len(valid_0_neg),
+                   len(valid_1_pos),
+                   len(valid_1_neg)))
 
         # TEST
         test_0_neg = all_0_neg
@@ -1969,8 +1981,15 @@ class CovariateDataset(Dataset):
         test_1_pos = all_1_pos
 
         if verbose:
-            print("TEST: neg={}, pos={}".format(len(test_0_neg)+len(test_1_neg),
-                                                len(test_0_pos)+len(test_1_pos)))
+            print("TEST (ratio={:.2}): neg={}, pos={}, d1_pos/neg={}/{}, d2_pos/neg={}/{}".format(
+                   1-ratio,
+                   len(test_0_neg)+len(test_1_neg),
+                   len(test_0_pos)+len(test_1_pos),
+                   len(test_0_pos),
+                   len(test_0_neg),
+                   len(test_1_pos),
+                   len(test_1_neg)))
+
 
         def _reduce_nsamples(nsamples, a, b, c, d):
             if nsamples:
