@@ -3,9 +3,12 @@ from collections import OrderedDict
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+import torchvision
 import urllib
 import pathlib
 import os
+import sys
+import requests
 import numpy as np
 import warnings; warnings.filterwarnings("ignore")
 
@@ -18,42 +21,60 @@ model_urls['all'] = {
     "op_threshs":[0.07422872, 0.038290843, 0.09814756, 0.0098118475, 0.023601074, 0.0022490358, 0.010060724, 0.103246614, 0.056810737, 0.026791653, 0.050318155, 0.023985857, 0.01939503, 0.042889766, 0.053369623, 0.035975814, 0.20204692, 0.05015312],
     "ppv80_thres":[0.72715247, 0.8885005, 0.92493945, 0.6527224, 0.68707734, 0.46127197, 0.7272054, 0.6127343, 0.9878492, 0.61979693, 0.66309816, 0.7853459, 0.930661, 0.93645346, 0.6788558, 0.6547198, 0.61614525, 0.8489876]
 }
+model_urls['densenet121-res224-all'] = model_urls['all']
+
 
 model_urls['nih'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/nih-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  'Atelectasis', 'Consolidation', 'Infiltration', 'Pneumothorax', 'Edema', 'Emphysema', 'Fibrosis', 'Effusion', 'Pneumonia', 'Pleural_Thickening', 'Cardiomegaly', 'Nodule', 'Mass', 'Hernia', '', '', '', '' ],
     "op_threshs":[0.039117552, 0.0034529066, 0.11396341, 0.0057298196, 0.00045666535, 0.0018880932, 0.012037827, 0.038744126, 0.0037213727, 0.014730946, 0.016149804, 0.054241467, 0.037198864, 0.0004403434, np.nan, np.nan, np.nan, np.nan],
 }
+model_urls['densenet121-res224-nih'] = model_urls['nih']
+
 
 model_urls['pc'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/pc-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  'Atelectasis', 'Consolidation', 'Infiltration', 'Pneumothorax', 'Edema', 'Emphysema', 'Fibrosis', 'Effusion', 'Pneumonia', 'Pleural_Thickening', 'Cardiomegaly', 'Nodule', 'Mass', 'Hernia', '', 'Fracture', '', '' ],
     "op_threshs": [0.031012505, 0.013347598, 0.081435576, 0.001262615, 0.002587246, 0.0035944257, 0.0023071, 0.055412333, 0.044385884, 0.042766232, 0.043258056, 0.037629247, 0.005658899, 0.0091741895, np.nan, 0.026507627, np.nan, np.nan]
 }
+model_urls['densenet121-res224-pc'] = model_urls['pc']
 
 model_urls['chex'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/chex-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  'Atelectasis', 'Consolidation', '', 'Pneumothorax', 'Edema', '', '', 'Effusion', 'Pneumonia', '', 'Cardiomegaly', '', '', '', 'Lung Lesion', 'Fracture', 'Lung Opacity', 'Enlarged Cardiomediastinum' ],
     "op_threshs": [0.1988969, 0.05710573, np.nan, 0.0531293, 0.1435217, np.nan, np.nan, 0.27212676, 0.07749717, np.nan, 0.19712369, np.nan, np.nan, np.nan, 0.09932402, 0.09273402, 0.3270967, 0.10888247],
 }
+model_urls['densenet121-res224-chex'] = model_urls['chex']
+
 
 model_urls['rsna'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/kaggle-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  '', '', '', '', '', '', '', '', 'Pneumonia', '', '', '', '', '', '', '', 'Lung Opacity', '' ],
     "op_threshs": [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, 0.13486601, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, 0.13511065, np.nan]
 }
+model_urls['densenet121-res224-rsna'] = model_urls['rsna']
 
 model_urls['mimic_nb'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/mimic_nb-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  'Atelectasis', 'Consolidation', '', 'Pneumothorax', 'Edema', '', '', 'Effusion', 'Pneumonia', '', 'Cardiomegaly', '', '', '', 'Lung Lesion', 'Fracture', 'Lung Opacity', 'Enlarged Cardiomediastinum' ],
     "op_threshs": [0.08558747, 0.011884617, np.nan, 0.0040595434, 0.010733786, np.nan, np.nan, 0.118761964, 0.022924708, np.nan, 0.06358637, np.nan, np.nan, np.nan, 0.022143636, 0.017476924, 0.1258702, 0.014020768],
 }
+model_urls['densenet121-res224-mimic_nb'] = model_urls['mimic_nb']
 
 model_urls['mimic_ch'] = {
     "weights_url":'https://github.com/mlmed/torchxrayvision/releases/download/v1/mimic_ch-densenet121-d121-tw-lr001-rot45-tr15-sc15-seed0-best.pt',
     "labels":[  'Atelectasis', 'Consolidation', '', 'Pneumothorax', 'Edema', '', '', 'Effusion', 'Pneumonia', '', 'Cardiomegaly', '', '', '', 'Lung Lesion', 'Fracture', 'Lung Opacity', 'Enlarged Cardiomediastinum' ],
     "op_threshs": [0.09121389, 0.010573786, np.nan, 0.005023008, 0.003698257, np.nan, np.nan, 0.08001232, 0.037242252, np.nan, 0.05006329, np.nan, np.nan, np.nan, 0.019866971, 0.03823637, 0.11303808, 0.0069147074],
 }
+model_urls['densenet121-res224-mimic_ch'] = model_urls['mimic_ch']
+
+model_urls['resnet50-res512-all'] = {
+    "description": 'This model was trained on the datasets pc-nih-rsna-siim-vin at a 512x512 resolution.',
+    "weights_url": 'https://github.com/mlmed/torchxrayvision/releases/download/v1/pc-nih-rsna-siim-vin-resnet50-test512-e400-state.pt',
+    "labels":[  'Atelectasis', 'Consolidation', 'Infiltration', 'Pneumothorax', 'Edema', 'Emphysema', 'Fibrosis', 'Effusion', 'Pneumonia', 'Pleural_Thickening', 'Cardiomegaly', 'Nodule', 'Mass', 'Hernia', 'Lung Lesion', 'Fracture', 'Lung Opacity', 'Enlarged Cardiomediastinum' ],
+    "op_threshs": [0.51570356, 0.50444704, 0.53787947, 0.50723547, 0.5025118, 0.5035252, 0.5038076, 0.51862943, 0.5078151, 0.50724894, 0.5056339, 0.510706, 0.5053923, 0.5020846, np.nan, 0.5080557, 0.5138526, np.nan],
+    "ppv80_thres": [0.690908, 0.720028, 0.7303882, 0.7235838, 0.6787441, 0.7304924, 0.73105824, 0.6839408, 0.7241559, 0.7219969, 0.6346738, 0.72764945, 0.7285066, 0.5735704, np.nan, 0.69684714, 0.7135549, np.nan]
+}
 
 class _DenseLayer(nn.Sequential):
     def __init__(self, num_input_features, growth_rate, bn_size, drop_rate):
@@ -110,7 +131,7 @@ class DenseNet(nn.Module):
     """
 
     def __init__(self, growth_rate=32, block_config=(6, 12, 24, 16), num_init_features=64, bn_size=4,
-                 drop_rate=0, num_classes=18, in_channels=1, weights=None, op_threshs=None, progress=True,
+                 drop_rate=0, num_classes=18, in_channels=1, weights=None, op_threshs=None, 
                  apply_sigmoid=False):
 
         super(DenseNet, self).__init__()            
@@ -165,20 +186,9 @@ class DenseNet(nn.Module):
                 
         # needs to be register_buffer here so it will go to cuda/cpu easily
         self.register_buffer('op_threshs', op_threshs)
-
-                
+ 
         if self.weights != None:
-            
-            url = model_urls[weights]["weights_url"]
-            weights_filename = os.path.basename(url)
-            weights_storage_folder = os.path.expanduser(os.path.join("~",".torchxrayvision","models_data"))
-            self.weights_filename_local = os.path.expanduser(os.path.join(weights_storage_folder,weights_filename))
-              
-            if not os.path.isfile(self.weights_filename_local):
-                print("Downloading weights...")
-                print("If this fails you can run `wget {} -O {}`".format(url, self.weights_filename_local))
-                pathlib.Path(weights_storage_folder).mkdir(parents=True, exist_ok=True)
-                download(url, self.weights_filename_local)
+            self.weights_filename_local = get_weights(weights)
             
             try:
                 savedmodel = torch.load(self.weights_filename_local, map_location='cpu')
@@ -190,13 +200,16 @@ class DenseNet(nn.Module):
                 self.load_state_dict(savedmodel.state_dict())
             except Exception as e:
                 print("Loading failure. Check weights file:", weights_filename_local)
-                raise(e)
+                raise e
             
             self.eval()
             
             if "op_threshs" in model_urls[weights]:
                 self.op_threshs = torch.tensor(model_urls[weights]["op_threshs"])
                 
+            self.upsample = nn.Upsample(size=(224, 224), mode='bilinear', align_corners=False)
+        
+                
     def __repr__(self):
         if self.weights != None:
             return "XRV-DenseNet121-{}".format(self.weights)
@@ -204,12 +217,18 @@ class DenseNet(nn.Module):
             return "XRV-DenseNet"
                 
     def features2(self, x):
+        
+        x = fix_resolution(x, 224, self)
+        
         features = self.features(x)
         out = F.relu(features, inplace=True)
         out = F.adaptive_avg_pool2d(out, (1, 1)).view(features.size(0), -1)
         return out
     
     def forward(self, x):
+        
+        x = fix_resolution(x, 224, self)
+        
         features = self.features2(x)
         out = self.classifier(features)
         
@@ -221,6 +240,105 @@ class DenseNet(nn.Module):
             out = op_norm(out, self.op_threshs)
         return out
 
+    
+##########################
+class ResNet(nn.Module):
+
+    def __init__(self, weights=None, apply_sigmoid=False):
+        super(ResNet, self).__init__()  
+        
+        self.weights = weights
+        self.apply_sigmoid = apply_sigmoid
+        
+        if weights is None:
+            raise Exception("Must specify the `weights` argument to load this model")
+        
+        self.weights_filename_local = get_weights(weights)
+        self.weights_dict = model_urls[weights]
+        self.pathologies = model_urls[weights]["labels"]
+
+        if self.weights.startswith("resnet101"):
+            self.model = torchvision.models.resnet101(num_classes=len(self.weights_dict["labels"]), pretrained=False)
+            #patch for single channel
+            self.model.conv1 = torch.nn.Conv2d(1, 64, kernel_size=7, stride=2, padding=3, bias=False)
+
+        elif self.weights.startswith("resnet50"):
+            self.model = torchvision.models.resnet50(num_classes=len(self.weights_dict["labels"]), pretrained=False)
+            #patch for single channel
+            self.model.conv1 = torch.nn.Conv2d(1, 64, kernel_size=7, stride=2, padding=3, bias=False)
+            
+        try:
+            self.model.load_state_dict(torch.load(self.weights_filename_local))
+        except Exception as e:
+            print("Loading failure. Check weights file:", self.weights_filename_local)
+            raise e
+        
+        if "op_threshs" in model_urls[weights]:
+            self.register_buffer('op_threshs', torch.tensor(model_urls[weights]["op_threshs"]))
+        
+        self.upsample = nn.Upsample(size=(512, 512), mode='bilinear', align_corners=False)
+        
+        self.eval()
+       
+    
+    def __repr__(self):
+        if self.weights != None:
+            return "XRV-ResNet-{}".format(self.weights)
+        else:
+            return "XRV-ResNet"
+    
+    def features(self, x):
+        
+        x = fix_resolution(x, 512, self)
+        
+        x = self.model.conv1(x)
+        x = self.model.bn1(x)
+        x = self.model.relu(x)
+        x = self.model.maxpool(x)
+
+        x = self.model.layer1(x)
+        x = self.model.layer2(x)
+        x = self.model.layer3(x)
+        x = self.model.layer4(x)
+
+        x = self.model.avgpool(x)
+        x = torch.flatten(x, 1)
+        return x
+    
+    def forward(self, x):
+        
+        x = fix_resolution(x, 512, self)
+        
+        out = self.model(x)
+        
+        if hasattr(self, 'apply_sigmoid') and self.apply_sigmoid:
+            out = torch.sigmoid(out)
+        
+        if hasattr(self,"op_threshs") and (self.op_threshs != None):
+            out = torch.sigmoid(out)
+            out = op_norm(out, self.op_threshs)
+        return out
+    
+
+
+warning_log = {}
+def fix_resolution(x, resolution, model):
+    """
+    Check resolution of input and resize to match requested
+    """
+    
+    # just skip it if upsample was removed somehow
+    if not hasattr(model, 'upsample') or (model.upsample == None):
+        return x
+    
+    if ((x.shape[2] != resolution) | (x.shape[3] != resolution)):
+        if not hash(model) in warning_log:
+            print("Warning: Input size ({}x{}) is not the native resolution ({}x{}) for this model. A resize will be performed but this could impact performance.".format(x.shape[2], x.shape[3], resolution, resolution))
+            warning_log[hash(model)] = True
+        return model.upsample(x)
+    return x
+    
+    
 def op_norm(outputs, op_threshs):
     """normalize outputs according to operating points for a given model.
     Args: 
@@ -232,8 +350,8 @@ def op_norm(outputs, op_threshs):
     #expand to batch size so we can do parallel comp
     op_threshs = op_threshs.expand(outputs.shape[0],-1)
     
-    #initial values will be 0
-    outputs_new = torch.zeros(outputs.shape, device = outputs.device)
+    #initial values will be 0.5
+    outputs_new = torch.zeros(outputs.shape, device = outputs.device)+0.5
     
     # only select non-nan elements otherwise the gradient breaks
     mask_leq = (outputs<op_threshs) & ~torch.isnan(op_threshs)
@@ -259,9 +377,33 @@ def get_densenet_params(arch):
         ret = dict(growth_rate=32, block_config=(6, 12, 24, 16), num_init_features=64)
     return ret
 
-                                      
-import sys
-import requests
+def get_model(weights, **kwargs):
+    if weights.startswith("densenet"):
+        return DenseNet(weights=weights, **kwargs)
+    elif weights.startswith("resnet"):
+        return ResNet(weights=weights, **kwargs)
+    else:
+        raise Exception("Unknown model")
+
+
+def get_weights(weights):
+    
+    if not weights in model_urls:
+        raise Exception("Weights not found. Valid options: {}".format(list(model_urls.keys())))
+    
+    url = model_urls[weights]["weights_url"]
+    weights_filename = os.path.basename(url)
+    weights_storage_folder = os.path.expanduser(os.path.join("~",".torchxrayvision","models_data"))
+    weights_filename_local = os.path.expanduser(os.path.join(weights_storage_folder,weights_filename))
+    
+    if not os.path.isfile(weights_filename_local):
+        print("Downloading weights...")
+        print("If this fails you can run `wget {} -O {}`".format(url, weights_filename_local))
+        pathlib.Path(weights_storage_folder).mkdir(parents=True, exist_ok=True)
+        download(url, weights_filename_local)
+    
+    return weights_filename_local
+    
 
 # from here https://sumit-ghosh.com/articles/python-download-progress-bar/
 def download(url, filename):
